from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
import pycx4.qcda as cda
from .menus.general_cm import CXGeneralCM


class CommonMixin:
    def __init__(self, **kwargs):
        self.chan = None
        self.status = 'unknown'
        self.def_stylesheet = None
        self.status_color = ''
        self.context_menu = None
        self.cx_connect()

    # this works for most widgets (which uses IChan)
    def cx_connect(self):
        if self._cname is None or self._cname == '':
            self.chan = None
            return
        self.chan = cda.IChan(self._cname, private=True, get_curval=True)
        self.chan.valueMeasured.connect(self.cs_update)
        self.chan.resolve.connect(self.resolve_proc)

    def cs_update(self, chan):
        if self.def_stylesheet is None:
            self.def_stylesheet = self.styleSheet()
        color, status = cda.rflags_color_status(chan.rflags)
        if self.status != status:
            if color is None:
                self.setStyleSheet(self.def_stylesheet)
            else:
                pass
                self.setStyleSheet("background-color:" + color)
            self.status = status
            self.status_color = color

    def resolve_proc(self, chan):
        if self.def_stylesheet is None:
            self.def_stylesheet = self.styleSheet()
        if chan.rslv_stat == cda.RslvStats.notfound:
            self.setStyleSheet("background-color:#404040;")
            self.status = 'not_found'
        elif chan.rslv_stat == cda.RslvStats.found:
            self.setStyleSheet(self.def_stylesheet)
            self.status = 'found'

    def contextMenuEvent(self, event):
        self.context_menu = CXGeneralCM(self)
        self.context_menu.popup(event.globalPos())

    @pyqtSlot(str)
    def set_cname(self, name):
        if self._cname == name:
            return
        self._cname = name
        self.cx_connect()

    def get_cname(self):
        return self._cname

    cname = pyqtProperty(str, get_cname, set_cname)





from cxwidgets.aQt.QtCore import Qt
from cxwidgets import BaseGridW, PSpinBox
from cxwidgets.aQt.QtWidgets import QLabel, QLineEdit, QWidgetAction
from .general_cm import CXGeneralCM


class CXSpinboxSettingsW(BaseGridW):
    def __init__(self, source_w, parent=None):
        super().__init__(parent)
        self.source_w = source_w
        self.l_h1 = QLabel("knob settings")
        self.l_h1.setStyleSheet("font-weight: bold; font-size: 12pt")
        self.grid.addWidget(self.l_h1, 0, 0, 1, 4, Qt.AlignHCenter)

        self.grid.addWidget(QLabel("step:"), 1, 0)
        self.step_sb = PSpinBox()
        self.grid.addWidget(self.step_sb, 1, 1)
        self.step_sb.setValue(source_w.singleStep())
        self.step_sb.done.connect(source_w.setSingleStep)

        self.grid.addWidget(QLabel("range:"), 2, 0)
        self.min_sb = PSpinBox()
        self.grid.addWidget(self.min_sb, 2, 1)
        self.min_sb.setValue(source_w.minimum())
        self.min_sb.done.connect(source_w.setMinimum)

        self.grid.addWidget(QLabel("-"), 2, 2)
        self.max_sb = PSpinBox()
        self.max_sb.setValue(source_w.maximum())
        self.max_sb.done.connect(source_w.setMaximum)
        self.grid.addWidget(self.max_sb, 2, 3)

        if not (source_w.hardmin == 0 and source_w.hardmax == 0):
            self.min_sb.setMinimum(source_w.hardmin)
            self.min_sb.setMaximum(source_w.hardmax)
            self.max_sb.setMinimum(source_w.hardmin)
            self.max_sb.setMaximum(source_w.hardmax)

        self.grid.addWidget(QLabel("max range:"), 3, 0)
        self.grid.addWidget(QLabel(str(source_w.hardmin)), 3, 1, Qt.AlignHCenter)
        self.grid.addWidget(QLabel("-"), 3, 2)
        self.grid.addWidget(QLabel(str(source_w.hardmax)), 3, 3, Qt.AlignHCenter)

        self.grid.setColumnMinimumWidth(1, 100)
        self.grid.setColumnMinimumWidth(3, 100)

        # ranges now working incorrectly - reported to Bolkhov
        # if source_w.chan is not None:
        #     source_w.chan.get_range()
        #     source_w.chan.get_strings()
        #     print(source_w.chan.quant)
        #     print(source_w.chan.rng)

    def validate_minmax(self):
        pass


class CXSpinboxCM(CXGeneralCM):
    def __init__(self, source_w):
        super().__init__(source_w)

        self.addSeparator()

        self.w1 = CXSpinboxSettingsW(source_w)
        self.act_set = QWidgetAction(self)
        self.act_set.setDefaultWidget(self.w1)
        self.addAction(self.act_set)




from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
from cxwidgets import PSwitch
import pycx4.qcda as cda
from .menus.general_cm import CXGeneralCM
from .common_mixin import CommonMixin

class CXSwitch(PSwitch, CommonMixin):
    def __init__(self, *args, **kwargs):
        self._cname = None
        super().__init__(*args, **kwargs)
        self._invert = kwargs.get('invert', False)
        self.done.connect(self.cs_send)

    def setInvert(self, inv):
        self._invert = inv

    def isInvert(self):
        return self._invert

    invert = pyqtProperty(bool, isInvert, setInvert)

    @pyqtSlot(bool)
    def cs_send(self, value):
        v = not bool(value) if self.invert else bool(value)
        if int(v) == self.chan.val:
            return
        self.chan.setValue(v)

    def cs_update(self, chan):
        super().cs_update(chan)
        v = not bool(chan.val) if self.invert else bool(chan.val)
        self.setValue(v)


class CXDevSwitch(PSwitch):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self.state_chan, self.off_chan, self.on_chan = None, None, None
        self._devname = kwargs.get('devname', None)
        self.cx_connect()
        self.done.connect(self.cs_send)

    def cx_connect(self):
        if self._devname is None:
            return
        self.state_chan = cda.IChan(self._devname + '.is_on', private=True, on_update=True)
        self.on_chan = cda.IChan(self._devname + '.switch_on', private=True, on_update=True)
        self.off_chan = cda.IChan(self._devname + '.switch_off', private=True, on_update=True)
        self.state_chan.valueChanged.connect(self.cs_update)

    @pyqtSlot(bool)
    def cs_send(self, value):
        if int(value) == self.state_chan.val:
            return
        if value:
            self.on_chan.setValue(1)
        else:
            self.off_chan.setValue(1)

    def cs_update(self, chan):
        self.setValue(bool(chan.val))

    @pyqtSlot(str)
    def set_devname(self, devname):
        if self._devname == devname:
            return
        self._devname = devname
        self.cx_connect()

    def get_devname(self):
        return self._devname

    devname = pyqtProperty(str, get_devname, set_devname)



#!/usr/bin/env python3

import sys
from aux.Qt import *
from cx_lcdnumber import CXLCDNumber
from cx_checkbox import CXCheckBox


app = QtWidgets.QApplication(sys.argv)
w = CXLCDNumber(None, 'canhw:13.BUN1.Imes')
w.show()

w2 = CXCheckBox(cname='Ql1.Iset')
w2.show()

sys.exit(app.exec_())



#!/usr/bin/env python3
import sys
from cxwidgets.aQt import QtWidgets, QtCore, QtGui
import pyqtgraph as pg


class BPMScopeItem(pg.GraphicsObject):
    def __init__(self, **kwargs):
        super().__init__()
        self.x_size = kwargs.get('xsize', 200)
        self.y_size = kwargs.get('ysize', 200)

        self.picture = None
        self.generatePicture()

    def generatePicture(self):
        self.picture = QtGui.QPicture()
        p = QtGui.QPainter(self.picture)
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)
        p.setPen(pg.mkPen('g'))
        p.drawEllipse(0, 0, self.x_size, self.y_size)
        p.drawEllipse(self.x_size/4, self.y_size/4, self.x_size/2, self.y_size/2)

        p.drawLine(self.x_size/2, 0, self.x_size/2, self.y_size)
        p.drawLine(0, self.y_size/2, self.x_size, self.y_size/2)

        p.end()

    def paint(self, p, *args):
        p.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        return QtCore.QRectF(self.picture.boundingRect())

class BeamItem(pg.GraphicsObject):
    def __init__(self, **kwargs):
        super().__init__()
        self.size = kwargs.get('size', 20)
        self.x0 = kwargs.get('x0', 0)
        self.y0 = kwargs.get('y0', 0)
        self.x = 0
        self.y = 0

        self.setBeamPos(0, 0)
        self._color = QtGui.QColor(255, 0, 0)
        self.picture = None
        self.generatePicture()

    def generatePicture(self):
        self.picture = QtGui.QPicture()
        p = QtGui.QPainter(self.picture)
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)

        #x = self.x0 + self.x
        #y = self.y0 + self.y
        r = self.size/2

        gradient = QtGui.QRadialGradient(r, r, r, r, r)
        gradient.setColorAt(0, QtCore.Qt.black)
        gradient.setColorAt(1, self._color)

        brush = QtGui.QBrush(gradient)
        p.setPen(self._color)
        p.setBrush(brush)

        p.drawEllipse(0, 0, self.size, self.size)

        p.end()

    def paint(self, p, *args):
        p.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        return QtCore.QRectF(self.picture.boundingRect())

    def setBeamPos(self, x, y):
        self.setPos(self.x0 - self.size/2 + x, self.y0 - self.size/2 - y)
        self.x, self.y = x, y



class BPMView(pg.GraphicsView):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self.x_size = kwargs.get('xsize', 200)
        self.y_size = kwargs.get('ysize', 200)

        self.resize(self.x_size, self.y_size)
        self.setAspectLocked(True)

        self.bpm_scope = BPMScopeItem(xsize=self.x_size, ysize=self.y_size)
        self.addItem(self.bpm_scope)

        self.beam = BeamItem(x0=self.x_size/2, y0=self.y_size/2)
        self.addItem(self.beam)

    def timer_move(self):
        self.setBeamXY(50, 50)

    def setBeamXY(self, x, y):
        self.beam.setBeamPos(x, y)


app = QtWidgets.QApplication(sys.argv)

## Create window with GraphicsView widget
w = BPMView(xsize=200, ysize=200)
w.show()

## Create image item
#img = pg.ImageItem(np.zeros((200, 200)))
#w.addItem(img)

## Set initial view bounds
#w.setRange(QtCore.QRectF(0, 0, 200, 200))

## start drawing with 3x3 brush
# kern = np.array([
#     [0.0, 0.5, 0.0],
#     [0.5, 1.0, 0.5],
#     [0.0, 0.5, 0.0]
# ])
# img.setDrawKernel(kern, mask=kern, center=(1,1), mode='add')
# img.setLevels([0, 10])

sys.exit(app.exec_())



from cxwidgets.aQt.QtCore import QTimer, Qt, QSize, pyqtSlot, pyqtProperty
from cxwidgets.aQt.QtGui import QColor, QPainter, QRadialGradient, QBrush
from cxwidgets.aQt.QtWidgets import QWidget


class LedWidget(QWidget):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self._diamX, self._diamY, self._diameter = 0, 0, 20
        self._on_color = QColor("red")
        self._off_color = self._on_color.darker(250)
        self._alignment = Qt.AlignCenter
        self._state = False
        self._timer = QTimer()
        self._timer.timeout.connect(self.toggleState)
        self.setDiameter(self._diameter)

    def paintEvent(self, event):
        d = self._diameter
        r = d / 2
        painter = QPainter()
        x = 0
        y = 0
        if self._alignment & Qt.AlignLeft:
            x = 0
        elif self._alignment & Qt.AlignRight:
            x = self.width() - d
        elif self._alignment & Qt.AlignHCenter:
            x = (self.width() - d) / 2
        elif self._alignment & Qt.AlignJustify:
            x = 0

        if self._alignment & Qt.AlignTop:
            y = 0
        elif self._alignment & Qt.AlignBottom:
            y = self.height() - d
        elif self._alignment & Qt.AlignVCenter:
            y = (self.height() - d) / 2

        color = self._on_color if self._state else self._off_color

        gradient = QRadialGradient(x + r, y + r, d * 0.4, d * 0.4, d * 0.4)
        gradient.setColorAt(0, Qt.white)
        gradient.setColorAt(1, color)

        painter.begin(self)
        brush = QBrush(gradient)
        painter.setPen(color)
        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.setBrush(brush)
        painter.drawEllipse(x, y, d - 1, d - 1)
        painter.end()

    def minimumSizeHint(self):
        return QSize(self._diameter, self._diameter)

    def sizeHint(self):
        return QSize(self._diameter + 1, self._diameter + 1)

    def getDiameter(self):
        return self._diameter

    @pyqtSlot(int)
    def setDiameter(self, value):
        self._diameter = value
        self.update()

    def getOnColor(self):
        return self._on_color

    @pyqtSlot(QColor)
    def setOnColor(self, value):
        self._on_color = value
        self.update()

    def getOffColor(self):
        return self._off_color

    @pyqtSlot(QColor)
    def setOffColor(self, value):
        self._off_color = value
        self.update()

    def getAlignment(self):
        return self._alignment

    @pyqtSlot(Qt.Alignment)
    def setAlignment(self, value):
        self._alignment = value
        self.update()

    def getState(self):
        return self._state

    @pyqtSlot(bool)
    def setState(self, value):
        self._state = value
        self.update()

    def getValue(self):
        return self._state

    @pyqtSlot(bool)
    def setValue(self, value):
        self._state = value
        self.update()

    @pyqtSlot()
    def toggleState(self):
        self._state = not self._state
        self.update()

    def flashOnce(self):
        self.toggleState()
        self._timer.singleShot(200, self.toggleState)

    diameter = pyqtProperty(int, getDiameter, setDiameter)
    on_color = pyqtProperty(QColor, getOnColor, setOnColor)
    off_color = pyqtProperty(QColor, getOffColor, setOffColor)
    alignment = pyqtProperty(Qt.Alignment, getAlignment, setAlignment)
    state = pyqtProperty(bool, getState, setState)
    value = pyqtProperty(bool, getValue, setValue)




from .auxwidgets import HLine, VLine, BaseGridW, BaseFrameGridW
from .pcheckbox import PCheckBox
from .pcombobox import PComboBox
from .pdoublespinbox import PDoubleSpinBox
from .plcdnumber import PLCDNumber
from .pspinbox import PSpinBox
from .pledwidget import LedWidget
from .pswitch import PSwitch

from .cx_doublespinbox import CXDoubleSpinBox
from .cx_spinbox import CXSpinBox
from .cx_lcdnumber import CXLCDNumber
from .cx_checkbox import CXCheckBox
from .cx_combobox import CXTextComboBox, CXIntComboBox
from .cx_pushbutton import CXPushButton
from .cx_lineedit import CXLineEdit
from .cx_progressbar import CXProgressBar
from .cx_switch import CXSwitch, CXDevSwitch
from .cx_label import CXIntLabel, CXDoubleLabel, CXStrLabel
from .cx_led import CXEventLed, CXStateLed


from .cx_bpm_plot import BPMWidget, K500BPMWidget

from .cx_plot import CXProcPlot, CXPlotDataItem, CXScrollPlotDataItem

__all__ = [HLine, VLine, BaseGridW, BaseFrameGridW, PCheckBox, PComboBox, PDoubleSpinBox, PLCDNumber,
           PSpinBox, LedWidget, PSwitch,
           CXCheckBox, CXTextComboBox, CXIntComboBox, CXDoubleSpinBox, CXLCDNumber, CXLineEdit, CXProgressBar,
           CXPushButton, CXSpinBox, BPMWidget, K500BPMWidget,
           CXSwitch, CXDevSwitch, CXEventLed, CXStateLed,
           CXIntLabel, CXDoubleLabel, CXStrLabel,
           CXProcPlot, CXPlotDataItem, CXScrollPlotDataItem
           ]




from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
from cxwidgets import PCheckBox
from .common_mixin import CommonMixin


class CXCheckBox(PCheckBox, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)
        self.done.connect(self.cs_send)

    @pyqtSlot(bool)
    def cs_send(self, value):
        if int(value) == self.chan.val:
            return
        self.chan.setValue(value)

    def cs_update(self, chan):
        super().cs_update(chan)
        self.setValue(bool(chan.val))






from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty, Qt
import pycx4.qcda as cda
from .pdoublespinbox import PDoubleSpinBox
from .menus.doublespinbox_cm import CXDoubleSpinboxCM
from .common_mixin import CommonMixin

class CXDoubleSpinBox(PDoubleSpinBox, CommonMixin):
    """Inherit: QDoubleSpinbox -> PDoubleSpinbox -> CXDoubleSpinBox.
       It is a double spinbox connected to CX channel.
       Gives some CX info and widget settings control over context menu.
       CX Channel created with data type corresponding to double.
       CX Channel is privately created in order to avoid any interference.

       When value set - it's send to CX (set considered by done signal of PDoubleSpinbox).
       When CX updates value - it's set to spinbox value.

       attributes:
           cname - cx channel name. When changed

    """
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)
        self.done.connect(self.cs_send)
        self.context_menu = None
        self._hardmin = 0
        self._hardmax = 0


    def contextMenuEvent(self, event):
        self.context_menu = CXDoubleSpinboxCM(self)
        self.context_menu.popup(event.globalPos())

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.DChan(self._cname, private=True, get_curval=True)
        self.chan.valueChanged.connect(self.cs_update)
        self.chan.resolve.connect(self.resolve_proc)

    @pyqtSlot(float)
    def cs_send(self, value):
        if self.chan is None:
            return
        if value != self.chan.val:
            self.chan.setValue(value)

    def cs_update(self, chan):
        super().cs_update(chan)
        if self.value() != chan.val:
            self.setValue(chan.val)

    @pyqtSlot(float)
    def set_hardmin(self, value):
        self._hardmin = value

    def get_hardmin(self):
        return self._hardmin

    hardmin = pyqtProperty(float, get_hardmin, set_hardmin)

    @pyqtSlot(float)
    def set_hardmax(self, value):
        self._hardmax = value

    def get_hardmax(self):
        return self._hardmax

    hardmax = pyqtProperty(float, get_hardmax, set_hardmax)




from cxwidgets.aQt.QtCore import Qt, pyqtSignal, pyqtProperty
from cxwidgets.aQt.QtWidgets import QDoubleSpinBox


class PDoubleSpinBox(QDoubleSpinBox):
    done = pyqtSignal(float)

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        # = kwargs.get("doneByEnter")

        self.valueChanged.connect(self.done)
        self.setRange(kwargs.get('min', -100000.0), kwargs.get('max', 100000.0))

        self.setFocusPolicy(Qt.StrongFocus)

        self._doneByEnter = False

    def wheelEvent(self, event):
        #print("focus", self.hasFocus())
        if self.hasFocus():
            super().wheelEvent(event)
        else:
            event.ignore()

    def focusInEvent(self, event):
        self.setFocusPolicy(Qt.WheelFocus)
        self.update()

    def focusOutEvent(self, event):
        self.setFocusPolicy(Qt.StrongFocus)
        self.update()

    @pyqtProperty(bool)
    def doneByEnter(self):
        return self._doneByEnter

    @doneByEnter.setter
    def doneByEnter(self, value):
        if self._doneByEnter == value:
            return
        if value:
            self.valueChanged.disconnect(self.done)
        else:
            self.valueChanged.disconnect(self.done)
        self._doneByEnter = value

    def keyPressEvent(self, event):
        if (event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter) and self._doneByEnter:
            self.done.emit(self.value())
        super().keyPressEvent(event)







# auxiliary widgets common for some programs

from cxwidgets.aQt.QtWidgets import QWidget, QGridLayout, QFrame

class HLine(QFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args)
        h = kwargs.get('h', 3)
        self.setFrameShape(QFrame.HLine)
        self.setFrameShadow(QFrame.Raised)
        self.setLineWidth(1)
        self.setMidLineWidth(h)


class VLine(QFrame):
    def __init__(self, *args, **kwargs):
        super().__init__(*args)
        h = kwargs.get('h', 3)
        self.setFrameShape(QFrame.VLine)
        self.setFrameShadow(QFrame.Raised)
        self.setLineWidth(1)
        self.setMidLineWidth(h)


class BaseGridW(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setContentsMargins(2, 2, 2, 2)
        self.grid = QGridLayout()
        self.grid.setContentsMargins(0, 0, 0, 0)
        self.grid.setSpacing(1)
        self.setLayout(self.grid)


class BaseFrameGridW(QFrame):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.setContentsMargins(2, 2, 2, 2)
        self.grid = QGridLayout()
        self.grid.setContentsMargins(0, 0, 0, 0)
        self.grid.setSpacing(1)
        self.setLayout(self.grid)

        self.setFrameStyle(QFrame.Sunken | QFrame.StyledPanel)
        self.setLineWidth(3)
        self.setMidLineWidth(3)



#!/usr/bin/env python3
from cxwidgets.aQt import QtWidgets, QtCore
import pycx4.qcda as cda
import pyqtgraph as pg
import numpy as np
import sys

# need to think some time







app = QtWidgets.QApplication(sys.argv)

# plt = pg.plot(np.random.normal(size=100), title="Simplest possible plotting example")
# print(plt)
w = CXHistoryPlot(cname='sled1.Imes', hist_time=5000)
#w.addChan('WG4_2.Imes')

w.show()


sys.exit(app.exec_())



from cxwidgets.aQt.QtWidgets import QLineEdit
from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
import pycx4.qcda as cda
from .common_mixin import CommonMixin

class CXLineEdit(QLineEdit, CommonMixin):
    def __init__(self, *args, **kwargs):
        self._cname = None
        self._max_len = kwargs.get('max_len', 100)
        super().__init__(*args, **kwargs)
        self.setReadOnly(bool(kwargs.get('readonly', False)))

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.StrChan(self._cname, max_nelems=self._max_len, private=True, on_update=True)
        self.chan.valueChanged.connect(self.cs_update)

    def cs_update(self, chan):
        super().cs_update(chan)
        if self.text() != chan.val:
            self.setText(chan.val)

    @pyqtSlot(str)
    def cs_send(self, value):
        self.chan.setValue(value)

    def set_max_len(self, ml):
        if self._max_len == ml:
            return
        self._max_len = ml
        self.cx_connect()

    def get_max_len(self):
        return self._max_len

    max_len = pyqtProperty(int, get_max_len, set_max_len)




from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
from .pspinbox import PSpinBox
from .menus.spinbox_cm import CXSpinboxCM
from .common_mixin import CommonMixin
import pycx4.qcda as cda


class CXSpinBox(PSpinBox, CommonMixin):
    def __init__(self, *args, **kwargs):
        self._cname = None
        super().__init__(*args, **kwargs)
        self.done.connect(self.cs_send)
        self._hardmin = 0
        self._hardmax = 0

    def contextMenuEvent(self, event):
        self.context_menu = CXSpinboxCM(self)
        self.context_menu.popup(event.globalPos())

    @pyqtSlot(int)
    def cs_send(self, value):
        if self.chan is None:
            return
        if value != self.chan.val:
            self.chan.setValue(value)

    def cs_update(self, chan):
        super().cs_update(chan)
        if self.value() != chan.val:
            self.setValue(chan.val)

    @pyqtSlot(int)
    def set_hardmin(self, value):
        self._hardmin = value

    def get_hardmin(self):
        return self._hardmin

    hardmin = pyqtProperty(int, get_hardmin, set_hardmin)

    @pyqtSlot(int)
    def set_hardmax(self, value):
        self._hardmax = value

    def get_hardmax(self):
        return self._hardmax

    hardmax = pyqtProperty(int, get_hardmax, set_hardmax)



#!/usr/bin/env python3

import sys
from PyQt5.QtWidgets import QApplication, QLabel
from cxwidgets.aQt.QtCore import Qt
from cxwidgets import BaseGridW, CXSwitch, CXDevSwitch


class DemoW(BaseGridW):
    def __init__(self, parent=None):
        super().__init__(parent)

        self.grid.addWidget(QLabel("CX widgets demo"), 0, 0, 1, 2, Qt.AlignHCenter)

        self.grid.addWidget(QLabel("CXSwitch"), 1, 0)
        s1 = CXSwitch(cname='cxhw:0.ddm.v2k_auto')
        self.grid.addWidget(s1, 1, 1)

        self.grid.addWidget(QLabel("CXDevSwitch"), 2, 0)
        s2 = CXDevSwitch(devname='cxhw:18.v0308_blm1.c0')
        self.grid.addWidget(s2, 2, 1)



app = QApplication(sys.argv)

grid = ()

#w = CXLCDNumber(None, 'canhw:13.BUN1.Imes')
#w.show()

#w2 = CXCheckBox(cname='Ql1.Iset')
#w2.show()

w = DemoW()

w.show()


sys.exit(app.exec_())



from cxwidgets.aQt.QtDesigner import QPyDesignerCustomWidgetPlugin
from cxwidgets.aQt.QtGui import QIcon
from cxwidgets import PDoubleSpinBox, PCheckBox, PLCDNumber, PSpinBox, LedWidget


class PDoubleSpinBoxWidgetPlugin(QPyDesignerCustomWidgetPlugin):
    def __init__(self, parent=None):
        super().__init__(parent)

    def name(self):
        return 'PDoubleSpinBox'

    def group(self):
        return 'polymorph widgets'

    def icon(self):
        return QIcon()

    def isContainer(self):
        return False

    def includeFile(self):
        return 'cxwidgets'

    def toolTip(self):
        return 'a double spinbox from polymorphs'

    def whatsThis(self):
        return 'a double spinbox from polymorphs'

    def createWidget(self, parent):
        return PDoubleSpinBox(parent)


class PComboBoxWidgetPlugin(PDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'PCheckBox'

    def toolTip(self):
        return 'checkbox from polymorphs'

    def whatsThis(self):
        return 'checkbox from polymorphs'

    def createWidget(self, parent):
        return PCheckBox(parent)


class PLCDNumberWidgetPlugin(PDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'PLCDNumber'

    def toolTip(self):
        return 'LCDNumber from polymorphs'

    def whatsThis(self):
        return 'LCDNumber from polymorphs'

    def createWidget(self, parent):
        return PLCDNumber(parent)


class PSpinBoxWidgetPlugin(PDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'PSpinBox'

    def toolTip(self):
        return 'spinbox from polymorphs'

    def whatsThis(self):
        return 'spinbox from polymorphs'

    def createWidget(self, parent):
        return PSpinBox(parent)


class LedPlugin(PDoubleSpinBoxWidgetPlugin):
    def name(self):
        return "LedWidget"

    def toolTip(self):
        return "LED-like"

    def whatsThis(self):
        return "LED-like"

    def createWidget(self, parent):
        return LedWidget(parent)




from PyQt5.QtWidgets import QPushButton, QSizePolicy
from PyQt5.QtGui import QBrush, QPen, QColor, QPainter
from PyQt5.QtCore import Qt, QRect, pyqtSignal, pyqtSlot, pyqtProperty


class PSwitch(QPushButton):
    done = pyqtSignal(bool)
    value = pyqtProperty(bool, QPushButton.isChecked)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setCheckable(True)
        self.setMinimumWidth(36)
        self.setMinimumHeight(18)
        size_policy = QSizePolicy(QSizePolicy.Preferred, QSizePolicy.Preferred)
        size_policy.setHeightForWidth(True)
        self.setSizePolicy(size_policy)
        self.a_ratio = 2.2
        self.toggled.connect(self.done)

    @pyqtSlot(bool)
    def setValue(self, value):
        self.setChecked(value)

    def heightForWidth(self, width):
        return width / self.a_ratio

    def paintEvent(self, event):
        bg_color = Qt.green if self.isChecked() else Qt.red
        pen_width = 2
        s = self.size()
        w0 = s.width()
        h0 = s.height()

        w = w0 - 2*pen_width - 2
        h = w/2 if w/2 < h0 - 2*pen_width else h0 - 2*pen_width
        r = h/3

        painter = QPainter(self)
        painter.setRenderHint(QPainter.Antialiasing)
        painter.translate(self.rect().center())
        painter.setBrush(QColor(130, 140, 160))

        pen = QPen(QColor(40, 40, 70))
        pen.setWidth(pen_width)
        painter.setPen(pen)
        painter.drawRoundedRect(QRect(-w/2, -h/2, w, h), r, r)

        painter.setBrush(QBrush(bg_color))
        sw_rect = QRect(-w/2, -h/2, h, h)
        if self.isChecked():
            sw_rect.moveLeft(w/2 - h)
        painter.drawRoundedRect(sw_rect, r, r)




from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
from cxwidgets.aQt.QtWidgets import QLabel
import pycx4.qcda as cda
from .common_mixin import CommonMixin

class CXIntLabel(QLabel, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)
        if self.chan is None:
            self.setText('No cname')

    def cs_update(self, chan):
        super().cs_update(chan)
        self.setText(str(chan.val))


class CXDoubleLabel(QLabel, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)
        if self.chan is None:
            self.setText('No cname')
        self._decimals = 0 # init stored
        self.decimals = 3 # default value

    def cs_update(self, chan):
        super().cs_update(chan)
        self.setText(format(chan.val, self.format_spec))

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.DChan(self._cname, private=True)
        self.chan.valueChanged.connect(self.cs_update)
        self.chan.resolve.connect(self.resolve_proc)

    @pyqtSlot(int)
    def setDecimals(self, n: int):
        self._decimals = n
        self.format_spec = '.' + str(self._decimals) + 'f'

    def getDecimals(self) -> int:
        return self._decimals

    decimals = pyqtProperty(int, getDecimals, setDecimals)


class CXStrLabel(QLabel, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        self._max_len = kwargs.get('max_len', 100)
        super().__init__(parent)

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.StrChan(self._cname, private=True, max_nelems=self._max_len, on_update=True)
        self.chan.valueChanged.connect(self.cs_update)
        self.chan.resolve.connect(self.resolve_proc)

    def cs_update(self, chan):
        super().cs_update(chan)
        self.setText(str(chan.val))

    @pyqtSlot(float)
    def set_max_len(self, max_len):
        self._max_len = max_len
        self.cx_connect()

    def get_max_len(self):
        return self._max_len

    max_len = pyqtProperty(int, get_max_len, set_max_len)




from cxwidgets.aQt.QtWidgets import QCheckBox
from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty, pyqtSignal


class PCheckBox(QCheckBox):
    done = pyqtSignal(bool)
    value = pyqtProperty(bool, QCheckBox.isChecked)

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self.clicked.connect(self.done)

    @pyqtSlot(bool)
    def setValue(self, state):
        self.setChecked(state)



#!/usr/bin/env python3
import sys
from cxwidgets.aQt import QtWidgets, QtCore, QtGui
from cxwidgets.aQt.QtCore import Qt
import pyqtgraph as pg
from cxwidgets.auxwidgets import BaseFrameGridW
import math


class BPMScopeItem(pg.GraphicsObject):
    def __init__(self, **kwargs):
        super().__init__()
        self.x_size = kwargs.get('xsize', 200)
        self.z_size = kwargs.get('zsize', 200)
        self.picture = None
        self.generatePicture()

    def generatePicture(self):
        self.picture = QtGui.QPicture()
        p = QtGui.QPainter(self.picture)
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)
        p.setPen(pg.mkPen('g'))
        p.drawEllipse(0, 0, self.x_size, self.z_size)
        p.drawEllipse(self.x_size/4, self.z_size/4, self.x_size/2, self.z_size/2)
        p.drawLine(self.x_size/2, 0, self.x_size/2, self.z_size)
        p.drawLine(0, self.z_size/2, self.x_size, self.z_size/2)
        p.end()

    def paint(self, p, *args):
        p.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        return QtCore.QRectF(self.picture.boundingRect())


class BPMGridItem(pg.GraphicsObject):
    def __init__(self, **kwargs):
        super().__init__()
        self.x_size = kwargs.get('xsize', 200)
        self.z_size = kwargs.get('zsize', 200)
        self.picture = None
        self.generatePicture()

    def generatePicture(self):
        self.picture = QtGui.QPicture()
        p = QtGui.QPainter(self.picture)
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)

        p.setPen(pg.mkPen(200, 200, 200))
        for ind in range(1, 10):
            p.drawLine(ind * self.x_size/10, 0, ind * self.x_size/10, self.z_size)
            p.drawLine(0, ind * self.z_size/10, self.x_size, ind * self.z_size/10)

        p.setPen(pg.mkPen(0, 0, 0))
        p.drawLine(self.x_size/2, 0, self.x_size/2, self.z_size)
        p.drawLine(0, self.z_size/2, self.x_size, self.z_size/2)
        p.end()

    def paint(self, p, *args):
        p.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        return QtCore.QRectF(self.picture.boundingRect())


class BeamItem(pg.GraphicsObject):
    def __init__(self, **kwargs):
        super().__init__()
        self.size = kwargs.get('size', 20)
        self.color = kwargs.get('color', "#ff0000")
        self.x0 = kwargs.get('x0', 0)
        self.z0 = kwargs.get('z0', 0)
        self.x = 0
        self.z = 0

        self.updateBeamPos()
        self._color = QtGui.QColor(self.color)
        self.picture = None
        self.generatePicture()

    def generatePicture(self):
        self.picture = QtGui.QPicture()
        p = QtGui.QPainter(self.picture)
        p.setRenderHint(QtGui.QPainter.Antialiasing, True)

        r = self.size/2
        gradient = QtGui.QRadialGradient(r, r, r, r, r)
        gradient.setColorAt(0, Qt.white)
        gradient.setColorAt(1, self._color)
        brush = QtGui.QBrush(gradient)
        p.setPen(self._color)
        p.setBrush(brush)
        p.drawEllipse(0, 0, self.size, self.size)
        p.end()

    def paint(self, p, *args):
        p.drawPicture(0, 0, self.picture)

    def boundingRect(self):
        return QtCore.QRectF(self.picture.boundingRect())

    def updateBeamPos(self):
        self.setPos(self.x0 - self.size/2 + self.x, self.z0 - self.size/2 - self.z)

    def setBeamPos(self, x, z):
        self.x, self.z = x, z
        self.updateBeamPos()


class BPMView(pg.GraphicsView):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self.x_size = kwargs.get('xsize', 200)
        self.z_size = kwargs.get('zsize', 200)
        self.x_max = kwargs.get('xmax', 20)
        self.z_max = kwargs.get('zmax', 20)
        self.use_scope = kwargs.get('use_scope', False)
        self.use_grid = kwargs.get('use_grid', True)
        self.beam_color = kwargs.get('beam_color', "#00ff00")

        self.setFixedSize(self.x_size, self.z_size)
        self.setAspectLocked(True)
        self.setBackground('w')

        if self.use_scope:
            self.bpm_scope = BPMScopeItem(xsize=self.x_size, zsize=self.z_size)
            self.addItem(self.bpm_scope)

        if self.use_grid:
            self.bpm_grid = BPMGridItem(xsize=self.x_size, zsize=self.z_size)
            self.addItem(self.bpm_grid)

        self.beam = BeamItem(x0=self.x_size/2, z0=self.z_size/2, color=self.beam_color)
        self.addItem(self.beam)

        self.beam_x, self.beam_z = 0, 0
        self.scale_x = self.x_size / (2.0 * self.x_max)
        self.scale_z = self.z_size / (2.0 * self.z_max)

    def setBeamXZ(self, x, z):
        self.beam.setBeamPos(x * self.scale_x, z * self.scale_z)

    def sizeHint(self):
        return QtCore.QSize(self.x_size + 5, self.z_size + 5)

    def minimumSizeHint(self):
        #return QtCore.QSize(self.x_size + 10, self.z_size + 10)
        return QtCore.QSize(self.x_size, self.z_size)


class BPMWidget(BaseFrameGridW):
    def __init__(self, parent=None, **kwargs):
        super(BPMWidget, self).__init__(parent)
        self.name = kwargs.get('label', 'BPM name')
        self.beam_color = kwargs.get('beam_color', "#00ff00")
        self.name_label = QtWidgets.QLabel(self.name)
        self.grid.addWidget(self.name_label, 0, 0, 1, 2)

        self.bpm_w = BPMView(xsize=180, zsize=180, xmax=20, zmax=20,
                             use_scope=False, use_grid=True, beam_color=self.beam_color)
        self.grid.addWidget(self.bpm_w, 1, 0)

        self.int_bar = QtWidgets.QProgressBar(orientation=Qt.Vertical)
        self.int_bar.setFixedWidth(25)
        self.int_bar.setStyleSheet("QProgressBar { text-align: center; color: #000000; font: 10px;}")
        self.grid.addWidget(self.int_bar, 1, 1)

        self.grid_l = QtWidgets.QGridLayout()
        self.grid.addLayout(self.grid_l, 2, 0, 1, 2)

        self.grid_l.addWidget(QtWidgets.QLabel("X:"), 0, 0, Qt.AlignRight)
        self.grid_l.addWidget(QtWidgets.QLabel("Z:"), 1, 0, Qt.AlignRight)
        self.x_label = QtWidgets.QLabel("0.00")
        self.z_label = QtWidgets.QLabel("0.00")
        self.grid_l.addWidget(self.x_label, 0, 1, Qt.AlignLeft)
        self.grid_l.addWidget(self.z_label, 1, 1, Qt.AlignLeft)

        self.grid_l.addWidget(QtWidgets.QLabel("cur:"), 0, 3, Qt.AlignRight)
        self.grid_l.addWidget(QtWidgets.QLabel("n:"), 1, 3, Qt.AlignRight)

        self.c_label = QtWidgets.QLabel("0.00")
        self.n_label = QtWidgets.QLabel("0.00")
        self.grid_l.addWidget(self.c_label, 0, 4, Qt.AlignLeft)
        self.grid_l.addWidget(self.n_label, 1, 4, Qt.AlignLeft)

        #self.setStyleSheet("background-color:#e0e0e0;")
        self.setStyleSheet("QLabel { font: bold 18px;}")

        self.beam_x, self.beam_z, self.prev_beam_x, self.prev_beam_z = 0, 0, 0, 0

    def setBeamXZ(self, x, z):
        self.bpm_w.setBeamXZ(x, z)
        self.x_label.setText("{:.2f}".format(x))
        self.z_label.setText("{:.2f}".format(z))

        self.prev_beam_x, self.prev_beam_z = self.beam_x, self.beam_z
        self.beam_x, self.beam_z = x, z
        delta = math.sqrt((self.prev_beam_x - x)**2 + (self.prev_beam_z - z)**2)
        if delta > 1:
            print('show last beam')

    def setImax(self, Imax):
        self.int_bar.setMaximum(Imax)

    def setIntensity(self, I_rel):
        self.int_bar.setValue(I_rel)

    def setCurrent(self, c):
        self.c_label.setText("{:.2f}".format(c))


import pycx4.qcda as cda


class K500BPM(QtCore.QObject):
    update_I = QtCore.pyqtSignal(float)
    update_x = QtCore.pyqtSignal(float)
    update_z = QtCore.pyqtSignal(float)
    update_c = QtCore.pyqtSignal(float)

    def __init__(self, name):
        super().__init__()
        self.name = name
        self.cnames = ['x', 'z', 'I', 'c']
        self.chans = {cn: cda.DChan(name + '.' + cn) for cn in self.cnames}
        for k in self.chans:
            self.chans[k].valueMeasured.connect(self.proc)

    def proc(self, chan):
        n = chan.short_name()
        getattr(self, "update_" + n).emit(chan.val)


class K500BPMWidget(BPMWidget):
    def __init__(self, name, **kwargs):
        super().__init__(**kwargs)
        self.name = name
        self.bpm = K500BPM(name)
        self.setImax(2*4*8192)

        self.bpm.update_I.connect(self.setIntensity)
        self.bpm.update_x.connect(self.new_beam_x)
        self.bpm.update_z.connect(self.new_beam_z)
        self.bpm.update_c.connect(self.setCurrent)

        self.beam_x, self.beam_z = 0, 0
        self.new_x, self.new_z = False, False

    def new_beam_x(self, x):
        self.beam_x = x
        if self.new_z:
            self.new_z = False
            self.setBeamXZ(self.beam_x, self.beam_z)
            return
        self.new_x = True

    def new_beam_z(self, z):
        self.beam_z = z
        if self.new_x:
            self.new_x = False
            self.setBeamXZ(self.beam_x, self.beam_z)
            return
        self.new_z = True


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)

    w = K500BPMWidget("cxout:3.k500.bpm.e2v2.6PIC1")
    w.show()

    sys.exit(app.exec_())



from cxwidgets.aQt.QtWidgets import QLCDNumber
from .common_mixin import CommonMixin


class CXLCDNumber(QLCDNumber, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)

    def cs_update(self, chan):
        super().cs_update(chan)
        self.display(chan.val)




# plot with processing capabilities

from PyQt5.QtWidgets import QCheckBox, QSpinBox, QDoubleSpinBox, QLabel, QPushButton,\
    QSpacerItem, QSizePolicy, QGroupBox, QGridLayout
from PyQt5.QtCore import QTimer

from .auxwidgets import BaseGridW, VLine
import json
import pyqtgraph as pg
import pycx4.qcda as cda
import time
import numpy as np


class CXPlotDataItem(pg.PlotDataItem):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._cname = kwargs.get('cname', None)
        self.max_nelems = kwargs.get('max_nelems', 4096)
        self.chan = cda.VChan(self._cname, max_nelems=self.max_nelems, private=True)
        self.chan.valueMeasured.connect(self.cs_update)

    def cs_update(self, chan):
        self.setData(chan.val)


# simple but not very optimized
class CXScrollPlotDataItem(pg.PlotDataItem):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._cname = kwargs.get('cname', None)
        self.length = kwargs.get('length', 1000)
        self.update_time = kwargs.get('update_time', 200)
        self.window = 0
        self.data = np.zeros(self.length)
        self.cur_data = self.data
        self.chan = cda.DChan(self._cname, private=True, on_update=True)
        self.chan.valueMeasured.connect(self.cs_update)
        self.n_update = 0

        self.setDownsampling(auto=True, method='peak')

        self.timer = QTimer()
        self.timer.start(self.update_time)
        self.timer.timeout.connect(self.plot_update)
        self.xd = np.arange(1 - 1 * self.length, 1)

    def cs_update(self, chan):
        self.cur_data[0] = chan.val
        self.cur_data = np.roll(self.cur_data, -1)
        if self.n_update < self.length:
            self.n_update += 1

    def plot_update(self):
        if self.n_update < self.length:
            self.setData(self.xd[-1 * self.n_update:], self.cur_data[-1 * self.n_update:])
        else:
            self.setData(self.xd, self.cur_data)




class CXPlotToolboxS(BaseGridW):
    def __init__(self):
        super().__init__()
        self.gb_general = QGroupBox('General processing')
        self.grid.addWidget(self.gb_general, 0, 0)
        self.gen_grid = QGridLayout()
        self.gb_general.setLayout(self.gen_grid)

        self.gen_grid.addWidget(QLabel('Averaging'), 0, 0)
        self.avg_spinbox = QSpinBox()
        self.avg_spinbox.setValue(1)
        self.avg_spinbox.setRange(1, 100)
        self.gen_grid.addWidget(self.avg_spinbox, 0, 1)

        self.getBgButton = QPushButton('get BG')
        self.gen_grid.addWidget(self.getBgButton, 1, 0)
        self.bg_checkbox = QCheckBox("apply bg")
        self.gen_grid.addWidget(self.bg_checkbox, 1, 1)

        self.clb_checkbox = QCheckBox("apply clb")
        self.gen_grid.addWidget(self.clb_checkbox, 2, 0)

        self.saveDataButton = QPushButton('Save data')
        self.gen_grid.addWidget(self.saveDataButton, 2, 1)

        # selected region tools
        self.gb_region = QGroupBox('Region processing')
        self.grid.addWidget(self.gb_region, 1, 0)
        self.reg_grid = QGridLayout()
        self.gb_region.setLayout(self.reg_grid)

        self.reg_checkbox = QCheckBox("proc region")
        self.reg_grid.addWidget(self.reg_checkbox, 0, 0)
        self.edges_label = QLabel("[0, 0]")
        self.reg_grid.addWidget(self.edges_label, 0, 1)

        self.reg_grid.addWidget(QLabel("reg avg"), 1, 0)
        self.reg_avg_label = QLabel("0.0")
        self.reg_grid.addWidget(self.reg_avg_label, 1, 1)

        self.grid.addItem(QSpacerItem(50, 500, vPolicy=QSizePolicy.Maximum), 2, 0)



class CXProcPlot(BaseGridW):
    def __init__(self, chan_or_name, sig_name='', t_step=1, calibr=None):
        super().__init__()
        self.sig_name = sig_name
        if isinstance(chan_or_name, cda.VPChan):
            self.chan = chan_or_name
        elif isinstance(chan_or_name, str):
            self.chan = cda.VPChan(chan_or_name, dtype=cda.DTYPE_SINGLE, max_nelems=783155, private=True)
        else:
            # need to raise exception here
            pass
        self.calibr = calibr
        self.t_step = t_step

        self.chan.valueMeasured.connect(self.update_plot)
        self.chan.bgReady.connect(self.bg_ready)

        self.draw_bg_val = False

        self.resize(1200, 800)

        self.toolbox = CXPlotToolboxS()
        self.grid.addWidget(self.toolbox, 0, 0)
        self.toolbox.avg_spinbox.valueChanged.connect(self.set_avg)
        self.toolbox.getBgButton.clicked.connect(self.get_bg)
        self.toolbox.bg_checkbox.stateChanged.connect(self.bg_apply)
        self.toolbox.saveDataButton.clicked.connect(self.save_data)

        self.toolbox.reg_checkbox.stateChanged.connect(self.region_proc)

        self.pen = (0, 255, 0)
        self.graph = pg.GraphicsLayoutWidget(parent=self)
        self.grid.addWidget(self.graph, 0, 1)
        self.plt = self.graph.addPlot(title=sig_name, autoDownsample=True)
        self.plt.disableAutoRange()
        self.plt.showGrid(x=True, y=True)
        self.curv = self.plt.plot(pen=self.pen)
        self.pen_avg = (100, 100, 100)
        self.curv_avg = self.plt.plot(pen=self.pen_avg)


    def update_plot(self, chan):
        v = chan.bg_val if self.draw_bg_val else chan.val
        if self.toolbox.clb_checkbox.isChecked() and self.sig_name in self.calibr:
            clb = self.calibr
            self.data = clb[0] * v * v + clb[1] * v
        else:
            self.data = v
        self.curv.setData(self.data)
        if self.toolbox.reg_checkbox.isChecked():
            self.toolbox.reg_avg_label.setText('{:2.5f}'.format(self.chan.reg_avg))

    def plot_avg(self, chan):
        self.curv_avg.setData(chan.avg_aval)

    def set_avg(self, n_avg):
        self.chan.setAveraging(n_avg)
        if n_avg == 1:
            self.curv_avg.clear()
        else:
            self.chan.avgReady.connect(self.plot_avg)

    def get_bg(self):
        self.chan.getBg(100)

    def bg_ready(self, chan):
        print('bg ready')

    def bg_apply(self, state):
        self.draw_bg_val = bool(state)
        self.chan.setBgApply(state)

    def save_data(self):
        print('saving data')
        out_data = {self.sig_name: self.data.tolist()}
        f = open(self.sig_name + "_" + time.strftime("%Y-%m-%d_%H-%M-%S") + ".json", 'w')
        json.dump(out_data, f)
        f.close()

    def region_proc(self, state):
        if state:
            self.lr = pg.LinearRegionItem([300, 1300])
            self.lr.setZValue(-10)
            self.lr.sigRegionChangeFinished.connect(self.region_changed)
            self.plt.addItem(self.lr)
            self.region_changed(self.lr)
        else:
            self.plt.removeItem(self.lr)

    def region_changed(self, l_reg):
        region = l_reg.getRegion()
        self.toolbox.edges_label.setText('[' + str(int(region[0])) + ', ' + str(int(region[1])) + ']')
        self.chan.setRegProc(int(region[0]), int(region[1]))


from cxwidgets.aQt.QtDesigner import QPyDesignerCustomWidgetPlugin
from cxwidgets.aQt.QtGui import QIcon
from cxwidgets import CXDoubleSpinBox, CXSpinBox, CXLCDNumber, CXCheckBox, CXTextComboBox
from cxwidgets import CXPushButton, CXLineEdit, CXProgressBar, CXSwitch, CXDevSwitch, CXIntLabel, CXDoubleLabel, CXStrLabel
from cxwidgets import CXEventLed, CXStateLed


class CXDoubleSpinBoxWidgetPlugin(QPyDesignerCustomWidgetPlugin):
    def __init__(self, parent=None):
        super(CXDoubleSpinBoxWidgetPlugin, self).__init__(parent)

    def name(self):
        return 'CXDoubleSpinBox'

    def group(self):
        return 'CX custom widgets'

    def icon(self):
        return QIcon()

    def isContainer(self):
        return False

    def includeFile(self):
        return 'cxwidgets'

    def toolTip(self):
        return 'double spinbox adapted to CX control system'

    def whatsThis(self):
        return 'double spinbox adapted to CX control system'

    def createWidget(self, parent):
        return CXDoubleSpinBox(parent)


class CXSpinBoxWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXSpinBox'

    def toolTip(self):
        return 'CXSpinBox connected to CX'

    def whatsThis(self):
        return 'CXSpinBox connected to CX'

    def createWidget(self, parent):
        return CXSpinBox(parent)


class CXLCDNumberWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXLCDNumber'

    def toolTip(self):
        return 'a LCDNumber connected to CX'

    def whatsThis(self):
        return 'a LCDNumber connected to CX'

    def createWidget(self, parent):
        return CXLCDNumber(parent)


class CXLCheckBoxWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXCheckBox'

    def toolTip(self):
        return 'CheckBox connected to CX'

    def whatsThis(self):
        return 'CheckBox connected to CX'

    def createWidget(self, parent):
        return CXCheckBox(parent)


class CXTextComboBoxWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXTextComboBox'

    def toolTip(self):
        return 'CXTextComboBox connected to CX'

    def whatsThis(self):
        return 'CXTextComboBox connected to CX'

    def createWidget(self, parent):
        return CXTextComboBox(parent)


class CXPushButtonWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXPushButton'

    def toolTip(self):
        return 'CXPushButton connected to CX'

    def whatsThis(self):
        return 'CXPushButton connected to CX'

    def createWidget(self, parent):
        return CXPushButton(parent)


class CXLineEditWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXLineEdit'

    def toolTip(self):
        return 'CXLineEdit connected to CX'

    def whatsThis(self):
        return 'CXLineEdit connected to CX'

    def createWidget(self, parent):
        return CXLineEdit(parent)


class CXProgressBarWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXProgressBar'

    def toolTip(self):
        return 'CXProgressBar connected to CX'

    def whatsThis(self):
        return 'CXProgressBar connected to CX'

    def createWidget(self, parent):
        return CXProgressBar(parent)


class CXSwitchWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXSwitch'

    def toolTip(self):
        return 'CXSwitch connected to CX'

    def whatsThis(self):
        return 'CXSwitch connected to CX'

    def createWidget(self, parent):
        return CXSwitch(parent)


class CXDevSwitchWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXDevSwitch'

    def toolTip(self):
        return 'CXDevSwitch connected to CX'

    def whatsThis(self):
        return 'CXDevSwitch connected to CX'

    def createWidget(self, parent):
        return CXDevSwitch(parent)


class CXEventLedWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXEventLed'

    def toolTip(self):
        return 'EventLed connected to CX'

    def whatsThis(self):
        return 'EventLed connected to CX'

    def createWidget(self, parent):
        return CXEventLed(parent)


class CXStateLedWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXStateLed'

    def toolTip(self):
        return 'StateLed connected to CX'

    def whatsThis(self):
        return 'StateLed connected to CX'

    def createWidget(self, parent):
        return CXStateLed(parent)


class CXIntLabelWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXIntLabel'

    def toolTip(self):
        return 'Label connected to CX int channel'

    def whatsThis(self):
        return 'Label connected to CX int channel'

    def createWidget(self, parent):
        return CXIntLabel(parent)


class CXDoubleLabelWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXDoubleLabel'

    def toolTip(self):
        return 'Label connected to CX double channel'

    def whatsThis(self):
        return 'Label connected to CX double channel'

    def createWidget(self, parent):
        return CXDoubleLabel(parent)


class CXStrLabelWidgetPlugin(CXDoubleSpinBoxWidgetPlugin):
    def name(self):
        return 'CXStrLabel'

    def toolTip(self):
        return 'Label connected to CX str channel'

    def whatsThis(self):
        return 'Label connected to CX str channel'

    def createWidget(self, parent):
        return CXStrLabel(parent)




from cxwidgets.aQt.QtWidgets import QProgressBar
from .common_mixin import CommonMixin

class CXProgressBar(QProgressBar, CommonMixin):
    def __init__(self, *args, **kwargs):
        self._cname = None
        super().__init__(*args, **kwargs)

    def cs_update(self, chan):
        super().cs_update(chan)
        if self.value() != chan.val:
            self.setValue(chan.val)




from cxwidgets.aQt.QtWidgets import QSpinBox
from cxwidgets.aQt.QtCore import pyqtSignal, Qt, pyqtProperty

class PSpinBox(QSpinBox):
    done = pyqtSignal(int)

    def __init__(self, *args, **kwargs):
        self._cname = None
        super().__init__(*args, **kwargs)
        self.valueChanged.connect(self.done)
        self.setRange(kwargs.get('min', -100000), kwargs.get('max', 100000))
        self._doneByEnter = False

    def wheelEvent(self, event):
        if self.hasFocus():
            super(PSpinBox, self).wheelEvent(event)
        else:
            event.ignore()

    def focusInEvent(self, event):
        self.setFocusPolicy(Qt.WheelFocus)
        self.update()

    def focusOutEvent(self, event):
        self.setFocusPolicy(Qt.StrongFocus)
        self.update()

    @pyqtProperty(bool)
    def doneByEnter(self):
        return self._doneByEnter

    @doneByEnter.setter
    def doneByEnter(self, value):
        if self._doneByEnter == value:
            return
        if value:
            self.valueChanged.disconnect(self.done)
        else:
            self.valueChanged.disconnect(self.done)
        self._doneByEnter = value

    def keyPressEvent(self, event):
        if (event.key() == Qt.Key_Return or event.key() == Qt.Key_Enter) and self._doneByEnter:
            self.done.emit(self.value())
        super().keyPressEvent(event)



from cxwidgets.aQt.QtWidgets import QPushButton
from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
import pycx4.qcda as cda
from .common_mixin import CommonMixin


class CXPushButton(QPushButton, CommonMixin):
    def __init__(self, *args, **kwargs):
        self._cname = None
        super().__init__(*args, **kwargs)
        self.clicked.connect(self.cs_send)

    @pyqtSlot()
    def cs_send(self):
        if self.chan is None:
            return
        self.chan.setValue(1)





from setuptools import setup, find_packages


setup(
    name='cxwidgets',
    version='0.12',
    author='Fedor Emanov',
    description='PyQt widgets connected to CX v4 control system framewok with designer plugins',

    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
    ],
    python_requires='>=3.5',
)


from cxwidgets.aQt.QtWidgets import QComboBox
from cxwidgets.aQt.QtGui import QIcon, QColor
from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty, Qt
import pycx4.qcda as cda
from .common_mixin import CommonMixin

class CXTextComboBox(QComboBox, CommonMixin):
    def __init__(self, *args, **kwargs):
        self._cname = None
        self._values = ['none'] + kwargs.get('values', [])
        self._colors = [None] + kwargs.get('colors', [])
        self._icons = kwargs.get('icons', None)
        self._max_len = kwargs.get('max_len', max([len(x) for x in self._values]))
        super().__init__(*args, **kwargs)

        for x in range(len(self._values)):
            self.insertItem(x, self._values[x])
            if self._icons is not None and x > 0:
                self.setItemIcon(x, QIcon(self._icons[x-1]))
            if x < len(self._colors):
                if self._colors[x] is not None:
                    self.setItemData(x, QColor(self._colors[x]), Qt.BackgroundRole)

        self.model().item(0).setEnabled(False)
        # keep next after all
        self.currentIndexChanged.connect(self.cs_send)
        self.currentIndexChanged.connect(self.update_bgcolor)

    def update_bgcolor(self, ind):
        if ind < len(self._colors):
            if self._colors[ind] is not None:
               self.setStyleSheet('QComboBox {background: ' + self._colors[ind] + ";}")

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.StrChan(self._cname, max_nelems=self._max_len, private=True)
        self.chan.valueChanged.connect(self.cs_update)
        self.chan.resolve.connect(self.resolve_proc)

    @pyqtSlot(str)
    def setValue(self, value):
        try:
            self.setCurrentIndex(self._values.index(value))
        except ValueError:
            self.setCurrentIndex(0)

    def value(self):
        return self._values[self.currentIndex()]

    @pyqtSlot(int)
    def cs_send(self, ind):
        if self.chan.val != self._values[ind]:
            self.chan.setValue(self._values[ind])

    def cs_update(self, chan):
        super().cs_update(chan)
        if self.value() != chan.val:
            self.setValue(chan.val)


class CXIntComboBox(QComboBox, CommonMixin):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._values = kwargs.get('values', {})

        for k in self._values:
            self.insertItem(k, self._values[k])

        self.currentIndexChanged.connect(self.cs_send)

    @pyqtSlot(str)
    def setValue(self, value):
        self.setCurrentIndex(value)

    def value(self):
        return self.currentIndex()

    @pyqtSlot(int)
    def cs_send(self, ind):
        if self.chan.val != ind:
            self.chan.setValue(ind)

    def cs_update(self, chan):
        if self.value() != chan.val:
            self.setValue(chan.val)




import sys
import importlib


qts = ['PyQt5', 'PyQt4']  ## ordered by preference

# check if PyQt alredy imported
QT_LIB = None

for lib in qts:
    if lib in sys.modules:
        QT_LIB = lib
        break

# if not imported let's try to import any
if QT_LIB is None:
    for lib in qts:
        try:
            importlib.import_module(lib)
            QT_LIB = lib
            break
        except ImportError:
            pass
if QT_LIB is None:
    ImportError("PyQt not found")

# now some PyQt is imported
PYQT5 = False
PYQT4 = False

if QT_LIB == 'PyQt5':
    PYQT5 = True

elif QT_LIB == 'PyQt4':
    PYQT4 = True




from cxwidgets.aQt.QtWidgets import QComboBox
from cxwidgets.aQt.QtCore import pyqtSignal

class PComboBox(QComboBox):

    done = pyqtSignal(int)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.currentIndexChanged.connect(self.done)
        self.cs_name = None
        self.val_list = None

    def setValue(self, val):
        if self.val_list:
            ind = self.val_list.index(val)
            self.setCurrentIndex(ind)
        else:
            self.setCurrentIndex(val)

    def value(self):
        return self.currentIndex()



from cxwidgets.aQt.QtCore import pyqtProperty, pyqtSlot
from .pledwidget import LedWidget
from .common_mixin import CommonMixin


class CXEventLed(LedWidget, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)
        self.setState(False)

    def cs_update(self, chan):
        super().cs_update(chan)
        self.flashOnce()


class CXStateLed(LedWidget, CommonMixin):
    def __init__(self, parent=None, **kwargs):
        self._cname = None
        super().__init__(parent, **kwargs)

    def cs_update(self, chan):
        super().cs_update(chan)
        self.setState(bool(chan.val))



from cxwidgets import BaseGridW, PSpinBox, PCheckBox, LedWidget, HLine
from cxwidgets.aQt.QtWidgets import QLabel, QLineEdit, QTextEdit, QMenu, QWidgetAction, QWidget, QHBoxLayout, QLayout
from cxwidgets.aQt.QtCore import Qt
from pycx4.qcda import rflags_order, rflags_meaning
import datetime
import math

class LabeledLed(QWidget):
    """LedWidget + label

    attributes:
        led - LedWidget
        label - Qlabel
    """
    def __init__(self, parent=None, text='label'):
        super().__init__()
        self.layout = QHBoxLayout(self)
        self.layout.setSpacing(0)
        self.layout.setSizeConstraint(QLayout.SetFixedSize)
        self.layout.setContentsMargins(0, 0, 0, 0)

        self.led = LedWidget(self)
        self.led.setDiameter(10)
        self.layout.addWidget(self.led)
        self.label = QLabel(text)
        self.layout.addWidget(self.label)


class CXGeneralCMW(BaseGridW):
    """Widget for general CX channel information
    """

    def __init__(self, source_widget, parent=None):
        super().__init__(parent)
        source_chan = source_widget.chan
        self.source_chan = source_chan
        self.l_h1 = QLabel("channel info")
        self.l_h1.setStyleSheet("font-weight: bold; font-size: 12pt")
        self.grid.addWidget(self.l_h1, 0, 0, 1, 2, Qt.AlignHCenter)
        self.grid.addWidget(HLine(), 1, 0, 1, 2, Qt.AlignHCenter)

        self.grid.addWidget(QLabel("name:  "), 2, 0)
        name_txt = 'n/a' if source_chan is None else source_chan.name
        self.grid.addWidget(QLabel(name_txt), 2, 1)

        self.grid.addWidget(QLabel("value:  "), 3, 0)
        self.label_val = QLabel('n/a')
        self.grid.addWidget(self.label_val, 3, 1)

        self.grid.addWidget(QLabel("time:  "), 4, 0)
        self.label_time = QLabel('n/a')
        self.grid.addWidget(self.label_time, 4, 1)

        self.grid.addWidget(QLabel("status:  "), 5, 0)
        self.label_status = QLabel(source_widget.status)
        self.grid.addWidget(self.label_status, 5, 1)

        if source_chan is not None:
            source_chan.valueMeasured.connect(self.data_update)
            self.data_update(source_chan)

    def data_update(self, chan):
        self.label_val.setText(str(chan.val))
        self.label_time.setText(datetime.datetime.fromtimestamp(chan.time / 1e6).strftime("%Y-%m-%d %H:%M:%S.%f"))



class CXFlagsMenu(QMenu):
    """QMenu to show CX rflags
    """
    def __init__(self, source_widget):
        super().__init__()
        source_chan = source_widget.chan
        self.source_chan = source_chan
        self.setTitle('rflags')
        self.items = {}
        self.widgets = {}

        for k in rflags_order:
            self.widgets[k] = LabeledLed(text=rflags_meaning[k])
            self.items[k] = QWidgetAction(self)
            self.items[k].setDefaultWidget(self.widgets[k])
            self.addAction(self.items[k])

        if source_chan is not None:
            self.setFlags(source_chan.rflags)

    def setFlags(self, rflags: int):
        for k in self.widgets:
            if (k & rflags) > 0:
                self.widgets[k].led.setValue(1)
            else:
                self.widgets[k].led.setValue(0)


class CXGeneralCM(QMenu):
    """General context menu for most CX-connected widgets

    """
    def __init__(self, source_w):
        super().__init__()
        self.source_w = source_w

        w = CXGeneralCMW(source_w)
        self.act_gen = QWidgetAction(self)
        self.act_gen.setDefaultWidget(w)
        self.addAction(self.act_gen)

        self.fl_menu = CXFlagsMenu(source_w)
        self.addMenu(self.fl_menu)

        # self.setAttribute(Qt.WA_DeleteOnClose) # this also works, but runs later
        self.aboutToHide.connect(self.deleteLater)
